From 224a915bd5af4139980993e7a011fa9a2dd4e6c7 Mon Sep 17 00:00:00 2001
From: Magnus Hagander <magnus@hagander.net>
Date: Thu, 11 Oct 2018 11:08:12 +0200
Subject: [PATCH 1/2] Slugify news URLs

This makes the URLs a lot nicer to view, and AFAIK (with no real
knowledge) better for search engines.
---
 pgweb/news/admin.py                 | 6 +++++-
 pgweb/news/feeds.py                 | 3 ++-
 pgweb/news/models.py                | 1 +
 pgweb/news/struct.py                | 3 ++-
 pgweb/news/views.py                 | 7 +++++--
 pgweb/urls.py                       | 3 ++-
 templates/index.html                | 8 ++++----
 templates/news/newsarchive.html     | 4 ++--
 templates/pages/developer/beta.html | 2 +-
 templates/pages/include/topbar.html | 2 +-
 templates/security/security.html    | 2 +-
 11 files changed, 26 insertions(+), 15 deletions(-)

diff --git a/pgweb/news/admin.py b/pgweb/news/admin.py
index 93ca0df..38c6ff3 100644
--- a/pgweb/news/admin.py
+++ b/pgweb/news/admin.py
@@ -1,10 +1,11 @@
 from django.contrib import admin
+from django.template.defaultfilters import slugify
 
 from pgweb.util.admin import PgwebAdmin
 from models import NewsArticle, NewsTag
 
 class NewsArticleAdmin(PgwebAdmin):
-	list_display = ('title', 'org', 'date', 'approved', )
+	list_display = ('title', 'org', 'date', 'approved', 'posturl')
 	list_filter = ('approved', )
 	filter_horizontal = ('tags', )
 	search_fields = ('content', 'title', )
@@ -17,6 +18,9 @@ class NewsArticleAdmin(PgwebAdmin):
 		}
 		return super(NewsArticleAdmin, self).change_view(request, object_id, extra_context=my_context)
 
+	def posturl(self, obj):
+		return '/news/{0}-{1}/'.format(slugify(obj.title), obj.id)
+
 class NewsTagAdmin(PgwebAdmin):
 	list_display = ('urlname', 'name', 'description')
 
diff --git a/pgweb/news/feeds.py b/pgweb/news/feeds.py
index 6c77fe3..88da2a5 100644
--- a/pgweb/news/feeds.py
+++ b/pgweb/news/feeds.py
@@ -1,4 +1,5 @@
 from django.contrib.syndication.views import Feed
+from django.template.defaultfilters import slugify
 
 from models import NewsArticle
 
@@ -21,7 +22,7 @@ class NewsFeed(Feed):
 			return NewsArticle.objects.filter(approved=True)[:10]
 
 	def item_link(self, obj):
-		return "https://www.postgresql.org/about/news/%s/" % obj.id
+		return "https://www.postgresql.org/about/news/{0}-{1}/".format(slugify(obj.title), obj.id)
 
 	def item_pubdate(self, obj):
 		return datetime.combine(obj.date,time.min)
diff --git a/pgweb/news/models.py b/pgweb/news/models.py
index 2c25489..911ae34 100644
--- a/pgweb/news/models.py
+++ b/pgweb/news/models.py
@@ -28,6 +28,7 @@ class NewsArticle(models.Model):
 
 	def purge_urls(self):
 		yield '/about/news/%s/' % self.pk
+		yield '/about/news/.*-%s/' % self.pk
 		yield '/about/newsarchive/'
 		yield '/news.rss'
 		yield '/news/.*.rss'
diff --git a/pgweb/news/struct.py b/pgweb/news/struct.py
index 2c06c87..e9a22c8 100644
--- a/pgweb/news/struct.py
+++ b/pgweb/news/struct.py
@@ -1,3 +1,4 @@
+from django.template.defaultfilters import slugify
 from datetime import date, timedelta
 from models import NewsArticle
 
@@ -13,5 +14,5 @@ def get_struct():
 		yearsold = (now - n.date).days / 365
 		if yearsold > 4:
 			yearsold = 4
-		yield ('about/news/%s/' % n.id,
+		yield ('about/news/{0}-{1}/'.format(slugify(n.title), n.id),
 			   0.5-(yearsold/10.0))
diff --git a/pgweb/news/views.py b/pgweb/news/views.py
index 94ac1e6..6491624 100644
--- a/pgweb/news/views.py
+++ b/pgweb/news/views.py
@@ -1,5 +1,6 @@
 from django.shortcuts import get_object_or_404
-from django.http import HttpResponse, Http404
+from django.http import HttpResponse, HttpResponseRedirect, Http404
+from django.template.defaultfilters import slugify
 from pgweb.util.decorators import login_required
 
 from pgweb.util.contexts import render_pgweb
@@ -23,10 +24,12 @@ def archive(request, tag=None, paging=None):
 		'newstags': NewsTag.objects.all(),
 	})
 
-def item(request, itemid, throwaway=None):
+def item(request, itemid, slug=None):
 	news = get_object_or_404(NewsArticle, pk=itemid)
 	if not news.approved:
 		raise Http404
+	if slug != slugify(news.title):
+		return HttpResponseRedirect('/about/news/{0}-{1}'.format(slugify(news.title), news.id))
 	return render_pgweb(request, 'about', 'news/item.html', {
 		'obj': news,
 		'newstags': NewsTag.objects.all(),
diff --git a/pgweb/urls.py b/pgweb/urls.py
index 93d01f1..741cf60 100644
--- a/pgweb/urls.py
+++ b/pgweb/urls.py
@@ -36,7 +36,8 @@ urlpatterns = [
 
 	url(r'^about/$', pgweb.core.views.about),
 	url(r'^about/newsarchive/([^/]+/)?$', pgweb.news.views.archive),
-	url(r'^about/news/(\d+)(-.*)?/$', pgweb.news.views.item),
+	url(r'^about/news/(?P<itemid>\d+)(?P<slug>-.*)?/$', pgweb.news.views.item),
+	url(r'^about/news/(?P<slug>[^/]+)-(?P<itemid>\d+)/$', pgweb.news.views.item),
 	url(r'^about/news/taglist.json/$', pgweb.news.views.taglist_json),
 	url(r'^about/events/$', pgweb.events.views.main),
 	url(r'^about/eventarchive/$', pgweb.events.views.archive),
diff --git a/templates/index.html b/templates/index.html
index 6c21c54..28dd14c 100644
--- a/templates/index.html
+++ b/templates/index.html
@@ -47,7 +47,7 @@
       <div class="text">
         <p>
           <strong>2018-09-20 - PostgreSQL 11 Beta 4 Released!</strong>
-          TThe PostgreSQL Global Development Group announces that the <a href="/about/news/1890/">fourth beta release of PostgreSQL 11</a> is now available for <a href="/download/">download</a>. This release contains <a href="/docs/11/static/release-11.html">previews of all features</a> that will be available in the final release of PostgreSQL 11, though <a href="/developer/beta/">some details of the release could change</a> before then.
+          TThe PostgreSQL Global Development Group announces that the <a href="/about/news/postgresql-11-beta-4-released-1890/">fourth beta release of PostgreSQL 11</a> is now available for <a href="/download/">download</a>. This release contains <a href="/docs/11/static/release-11.html">previews of all features</a> that will be available in the final release of PostgreSQL 11, though <a href="/developer/beta/">some details of the release could change</a> before then.
         </p>
         <ul>
           {% for v in versions %}
@@ -129,7 +129,7 @@
       <h2 class="centered-lines">Latest News</h2>
       <!-- Featured Post -->
         <header>
-          <h3><a href="/about/news/1890/">PostgreSQL 11 Beta 4 Released!</a></h3>
+          <h3><a href="/about/news/postgresql-11-beta-4-released-1890/">PostgreSQL 11 Beta 4 Released!</a></h3>
           <ul class="meta">
             <li><i class="far fa-clock"></i>&nbsp;2018-09-20</li>
             <!--<li><i class="far fa-comments"></i> 0</li>-->
@@ -139,7 +139,7 @@
           The PostgreSQL Global Development Group announces that the fourth beta release of PostgreSQL 11 is now available for download. This release contains previews of all features that will be available in the final release of PostgreSQL 11, though some details of the release could change before then.
         </p>
         <ul class="bold">
-          <li><a href="/about/news/1890/">Release Announcement</a></li>
+          <li><a href="/about/news/postgresql-11-beta-4-released-1890/">Release Announcement</a></li>
           <li><a href="/docs/11/static/release-11.html">PostgreSQL 11 Beta 4 Release Notes</a></li>
           <li><a href="/download/">Download</a></li>
         </ul>
@@ -152,7 +152,7 @@
         <ul class="divided">
           {% for n in news %}
             <li>
-              <h3><a href="/about/news/{{ n.id }}/">{{ n.title }}</a></h3>
+              <h3><a href="/about/news/{{ n.title|slugify }}-{{ n.id }}/">{{ n.title }}</a></h3>
               <ul class="meta">
                 <li><i class="far fa-clock"></i>&nbsp;{{ n.displaydate }} by {{ n.org.name }}</li>
               </ul>
diff --git a/templates/news/newsarchive.html b/templates/news/newsarchive.html
index e5dd3b3..10bc559 100644
--- a/templates/news/newsarchive.html
+++ b/templates/news/newsarchive.html
@@ -9,10 +9,10 @@
 </p>
 
 {%for obj in news %}
-<h2 class="news"><a href="/about/news/{{obj.id}}/">{{obj.title}}</a></h2>
+<h2 class="news"><a href="/about/news/{{obj.title|slugify}}-{{obj.id}}/">{{obj.title}}</a></h2>
 <div class="newsdate">Posted on <strong>{{obj.displaydate}}</strong>{% if obj.org.name != '_migrated' %} by {{ obj.org.name }}{% endif %}</div>
 {{obj.content|markdown|striptags|truncatewords:20}}
-<p><a href="/about/news/{{obj.id}}/">Read more...</a></p>
+<p><a href="/about/news/{{obj.title|slugify}}-{{obj.id}}/">Read more...</a></p>
 {%endfor%}
 <p><a href="/account/news/new/">Submit news</a></p>
 {%endblock%}
diff --git a/templates/pages/developer/beta.html b/templates/pages/developer/beta.html
index d2c4d3c..f73f3ea 100644
--- a/templates/pages/developer/beta.html
+++ b/templates/pages/developer/beta.html
@@ -38,7 +38,7 @@
 
 <ul>
   <li>
-    <a href="/about/news/1890/">PostgreSQL 11 Beta 4 Release Announcement</a>
+    <a href="/about/news/postgresql-11-beta-4-released-1890/">PostgreSQL 11 Beta 4 Release Announcement</a>
   </li>
   <li>
     <a href="/ftp/source/v11beta4/">Download 11 Beta 4 source code</a>
diff --git a/templates/pages/include/topbar.html b/templates/pages/include/topbar.html
index 4f697b1..be32820 100644
--- a/templates/pages/include/topbar.html
+++ b/templates/pages/include/topbar.html
@@ -1 +1 @@
-20th September 2018: <a href="{{link_root}}/about/news/1890/">PostgreSQL 11 Beta 4 Released</a>!
+20th September 2018: <a href="{{link_root}}/about/news/postgresql-11-beta-4-released-1890/">PostgreSQL 11 Beta 4 Released</a>!
diff --git a/templates/security/security.html b/templates/security/security.html
index 1466fc3..a6e0a6f 100644
--- a/templates/security/security.html
+++ b/templates/security/security.html
@@ -76,7 +76,7 @@ You can filter the view of patches to show just patches for version:<br/>
       <tr>
         <td>
           {%if p.cve%}<nobr>{%if p.cve_visible%}<a href="{{p.cvelink}}">CVE-{{p.cve}}</a>{%else%}CVE-{{p.cve}}{%endif%}</nobr><br/>{%endif%}
-          {%if p.newspost%}<a href="/about/news/{{p.newspost.id}}/">Announcement</a><br/>{%endif%}
+          {%if p.newspost%}<a href="/about/news/{{p.newspost.title|slugify}}-{{p.newspost.id}}/">Announcement</a><br/>{%endif%}
         </td>
         <td>{{p.affected|join:", "}}</td>
         <td>{{p.fixed|join:", "}}</td>
-- 
2.11.0

